'use strict';

exports.__esModule = true;

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _internalAttributes = require('../processing/dom/internal-attributes');

var _internalAttributes2 = _interopRequireDefault(_internalAttributes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var SOURCE_MAP_REG_EX = /#\s*sourceMappingURL\s*=\s*[^\s]+(\s|\*\/)/i; // -------------------------------------------------------------
// WARNING: this file is used by both the client and the server.
// Do not use any browser or node-specific API!
// -------------------------------------------------------------
/* eslint hammerhead/proto-methods: 2 */

var CSS_URL_PROPERTY_VALUE_PATTERN = /(url\s*\(\s*)(?:(')([^\s']*)(')|(")([^\s"]*)(")|([^\s\)]*))(\s*\))|(@import\s+)(?:(')([^\s']*)(')|(")([^\s"]*)("))/g;

var StyleProcessor = function () {
    function StyleProcessor() {
        (0, _classCallCheck3.default)(this, StyleProcessor);

        this.IS_STYLESHEET_PROCESSED_COMMENT = '/* stylesheet processed via hammerhead */';
    }

    StyleProcessor.prototype.process = function process(css, urlReplacer, isStylesheetTable) {
        var isStyleSheetProcessingRegEx = new RegExp('^\\s*' + this.IS_STYLESHEET_PROCESSED_COMMENT.replace(/\/|\*/g, '\\$&'));
        var isStylesheetProcessed = isStyleSheetProcessingRegEx.test(css);

        if (typeof css === 'string' && !isStylesheetProcessed) {
            var prefix = isStylesheetTable ? this.IS_STYLESHEET_PROCESSED_COMMENT + '\n' : '';

            // NOTE: Replace the :hover pseudo-class.
            css = css.replace(/\s*:\s*hover(\W)/gi, '[' + _internalAttributes2.default.hoverPseudoClass + ']$1');

            // NOTE: Remove the ‘source map’ directive.
            css = css.replace(SOURCE_MAP_REG_EX, '$1');

            // NOTE: Replace URLs in CSS rules with proxy URLs.
            return prefix + this._replaceStylsheetUrls(css, urlReplacer);
        }

        return css;
    };

    StyleProcessor.prototype.cleanUp = function cleanUp(css, parseProxyUrl) {
        if (typeof css === 'string') {
            css = css.replace(new RegExp('\\[' + _internalAttributes2.default.hoverPseudoClass + '\\](\\W)', 'ig'), ':hover$1').replace(new RegExp('^\\s*' + this.IS_STYLESHEET_PROCESSED_COMMENT.replace(/\/|\*/g, '\\$&') + '\n?'), '');

            return this._replaceStylsheetUrls(css, function (url) {
                var parsedProxyUrl = parseProxyUrl(url);

                return parsedProxyUrl ? parsedProxyUrl.destUrl : url;
            });
        }

        return css;
    };

    StyleProcessor.prototype._replaceStylsheetUrls = function _replaceStylsheetUrls(css, processor) {
        return css.replace(CSS_URL_PROPERTY_VALUE_PATTERN, function (match, prefix1, openQuote1, url1, closeQuote1, openQuote2, url2, closeQuote2, url3, postfix, prefix2, openQuote3, url4, closeQuote3, openQuote4, url5, closeQuote4) {
            var prefix = prefix1 || prefix2;
            var openQuote = openQuote1 || openQuote2 || openQuote3 || openQuote4 || '';
            var url = url1 || url2 || url3 || url4 || url5;
            var closeQuote = closeQuote1 || closeQuote2 || closeQuote3 || closeQuote4 || '';

            postfix = postfix || '';

            return url ? prefix + openQuote + processor(url) + closeQuote + postfix : match;
        });
    };

    return StyleProcessor;
}();

exports.default = new StyleProcessor();
module.exports = exports['default'];