'use strict';

exports.__esModule = true;

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _mustache = require('mustache');

var _mustache2 = _interopRequireDefault(_mustache);

var _shortid = require('shortid');

var _shortid2 = _interopRequireDefault(_shortid);

var _readFileRelative = require('read-file-relative');

var _events = require('events');

var _cookies = require('./cookies');

var _cookies2 = _interopRequireDefault(_cookies);

var _storage = require('../upload/storage');

var _storage2 = _interopRequireDefault(_storage);

var _command = require('./command');

var _command2 = _interopRequireDefault(_command);

var _url = require('../utils/url');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Const
var TASK_TEMPLATE = (0, _readFileRelative.readSync)('../client/task.js.mustache');

// Session

var Session = function (_EventEmitter) {
    (0, _inherits3.default)(Session, _EventEmitter);

    function Session(uploadsRoot) {
        (0, _classCallCheck3.default)(this, Session);

        var _this = (0, _possibleConstructorReturn3.default)(this, _EventEmitter.call(this));

        _this.uploadStorage = new _storage2.default(uploadsRoot);

        _this.id = Session._generateSessionId();
        _this.cookies = new _cookies2.default();
        _this.proxy = null;
        _this.pageLoadCount = 0;
        _this.injectable = {
            scripts: ['/hammerhead.js'],
            styles: []
        };
        return _this;
    }

    Session._generateSessionId = function _generateSessionId() {
        // NOTE: GH-116
        return _shortid2.default.generate();
    };

    Session.prototype.handleServiceMessage = function () {
        var ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(msg, serverInfo) {
            return _regenerator2.default.wrap(function _callee$(_context) {
                while (1) {
                    switch (_context.prev = _context.next) {
                        case 0:
                            if (!this[msg.cmd]) {
                                _context.next = 4;
                                break;
                            }

                            _context.next = 3;
                            return this[msg.cmd](msg, serverInfo);

                        case 3:
                            return _context.abrupt('return', _context.sent);

                        case 4:
                            throw new Error('Malformed service message or message handler is not implemented');

                        case 5:
                        case 'end':
                            return _context.stop();
                    }
                }
            }, _callee, this);
        }));

        function handleServiceMessage(_x, _x2) {
            return ref.apply(this, arguments);
        }

        return handleServiceMessage;
    }();

    Session.prototype._fillTaskScriptTemplate = function _fillTaskScriptTemplate(serverInfo, isFirstPageLoad, referer, cookie, iframeTaskScriptTemplate, payloadScript) {
        referer = referer === null ? '{{{referer}}}' : referer;
        cookie = cookie === null ? '{{{cookie}}}' : cookie;

        iframeTaskScriptTemplate = iframeTaskScriptTemplate === null ? '{{{iframeTaskScriptTemplate}}}' : iframeTaskScriptTemplate;

        return _mustache2.default.render(TASK_TEMPLATE, {
            sessionId: this.id,
            serviceMsgUrl: serverInfo.domain + '/messaging',
            ie9FileReaderShimUrl: serverInfo.domain + '/ie9-file-reader-shim',
            crossDomainPort: serverInfo.crossDomainPort,
            isFirstPageLoad: isFirstPageLoad,
            referer: referer,
            cookie: cookie,
            iframeTaskScriptTemplate: iframeTaskScriptTemplate,
            payloadScript: payloadScript
        });
    };

    Session.prototype.getIframeTaskScriptTemplate = function getIframeTaskScriptTemplate(serverInfo) {
        return (0, _stringify2.default)(this._fillTaskScriptTemplate(serverInfo, false, null, null, null, this._getIframePayloadScript(true)));
    };

    Session.prototype.getTaskScript = function getTaskScript(referer, cookieUrl, serverInfo, isIframe, withPayload) {
        var cookies = (0, _stringify2.default)(this.cookies.getClientString(cookieUrl));
        var payloadScript = '';

        if (withPayload) payloadScript = isIframe ? this._getIframePayloadScript() : this._getPayloadScript();

        var taskScript = this._fillTaskScriptTemplate(serverInfo, this.pageLoadCount === 0, referer, cookies, this.getIframeTaskScriptTemplate(serverInfo), payloadScript);

        this.pageLoadCount++;

        return taskScript;
    };

    Session.prototype._getIframePayloadScript = function _getIframePayloadScript() /* iframeWithoutSrc */{
        throw new Error('Not implemented');
    };

    Session.prototype._getPayloadScript = function _getPayloadScript() {
        throw new Error('Not implemented');
    };

    Session.prototype.handleFileDownload = function handleFileDownload() /* ctx */{
        throw new Error('Not implemented');
    };

    Session.prototype.handlePageError = function handlePageError() /* ctx, err */{
        throw new Error('Not implemented');
    };

    Session.prototype.getAuthCredentials = function getAuthCredentials() {
        throw new Error('Not implemented');
    };

    return Session;
}(_events.EventEmitter);

// Service message handlers


exports.default = Session;
var ServiceMessages = Session.prototype;

ServiceMessages[_command2.default.setCookie] = function (msg) {
    var parsedUrl = (0, _url.parseProxyUrl)(msg.url);
    var cookieUrl = parsedUrl ? parsedUrl.destUrl : msg.url;

    this.cookies.setByClient(cookieUrl, msg.cookie);

    return this.cookies.getClientString(cookieUrl);
};

ServiceMessages[_command2.default.uploadFiles] = function () {
    var ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(msg) {
        return _regenerator2.default.wrap(function _callee2$(_context2) {
            while (1) {
                switch (_context2.prev = _context2.next) {
                    case 0:
                        _context2.next = 2;
                        return this.uploadStorage.store(msg.fileNames, msg.data);

                    case 2:
                        return _context2.abrupt('return', _context2.sent);

                    case 3:
                    case 'end':
                        return _context2.stop();
                }
            }
        }, _callee2, this);
    }));
    return function (_x3) {
        return ref.apply(this, arguments);
    };
}();

ServiceMessages[_command2.default.getUploadedFiles] = function () {
    var ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3(msg) {
        return _regenerator2.default.wrap(function _callee3$(_context3) {
            while (1) {
                switch (_context3.prev = _context3.next) {
                    case 0:
                        _context3.next = 2;
                        return this.uploadStorage.get(msg.filePaths);

                    case 2:
                        return _context3.abrupt('return', _context3.sent);

                    case 3:
                    case 'end':
                        return _context3.stop();
                }
            }
        }, _callee3, this);
    }));
    return function (_x4) {
        return ref.apply(this, arguments);
    };
}();
module.exports = exports['default'];